/////////////////////////////////////////////////////////////////////
// Project:         Password Revealer v1.0 (prjPWREV)
// Programmer:      Nathan Factor
// E-Mail Addr:     nfactor99@yahoo.com
// Written In:      Borland Delphi 5.0 (Pro)
// Date:            August 09, 2000
// Level:           Advanced
// Description:     This project demonstrates how to create a program
//                  without the overhead of the Delphi VCL by using
//                  pure API (the executable itself is only 20kb!).
//                  The application itself can be used to reveal a
//                  password that is stored in an Edit control from
//                  behind the asterisks (e.g. '*****').
// Other Notes:     This project assumes that you possess a fairly
//                  solid understanding of the Windows API. However,
//                  if you are still unclear about some of the API
//                  calls, you can always search the MSDN library
//                  (http://msdn.microsoft.com) for CreateWindow, and
//                  CreateWindowEx, as well as the other calls used
//                  in this project.
/////////////////////////////////////////////////////////////////////


program prjPWREV;

uses
  Windows,
  Messages;

var
  Msg : TMsg;
  wClass : TWndClassA;
  intScrX, intScrY : integer;
  hInst, hFont, hMainWin, hBevel, hEdit,
  hLabel, hBtnAbt, hBtnHlp, hLastWin : HWND;

{$R *.RES}


// EndProgramProc - frees memory/resources when the
// user chooses to exit the application
procedure EndProgramProc();
begin
  // Free system resources associated with our font handle
  DeleteObject(hFont);
  // Remove our window class from memory
  UnRegisterClass('NF_PW_REVEALER', hInst);
  // End the process and all its threads 
  ExitProcess(hInst);
end;


// ShowAboutProc - displays a custom message box, using the
// MessageBoxIndirect API call, with the application's main icon
procedure ShowAboutProc();
var
  msgPars : TMsgBoxParams;
begin
  with msgPars do
    begin
      // Define the parameters of the message box
      cbSize := SizeOf(msgPars);
      hwndOwner := hMainWin;
      hInstance := hInst;
      lpszText := 'Password Revealer v1.0' + #13#10 +
                    'Coded in Delphi 5 (Pro) by Nathan Factor   ' +
                    #13#10#13#10 + 'This application is freeware!';
      lpszCaption := 'About Password Revealer';
      dwStyle := MB_OK or MB_USERICON;
      lpszIcon := 'MAINICON';
      dwContextHelpId := 0;
      lpfnMsgBoxCallback := nil;
      dwLanguageId := LANG_NEUTRAL;
    end;
  // Create the custom message box using the paramaters
  // that we have defined above (msgPars)
  MessageBoxIndirect(msgPars);
end;


// ShowHelpProc - displays a message box containing basic
// information on how to use the application
procedure ShowHelpProc();
begin
  MessageBox(hMainWin, 'Place the mouse cursor over a password ' +
              'field and the hidden password will appear in the ' +
              'text box of this program... revealed!',
              'Password Revealer Help', MB_OK or MB_ICONINFORMATION);
end;


// TimerProc - processes the WM_TIMER messages received by our
// program; executes code to retrieve a password from an Edit
// control from behind the asterisks (e.g. '******')
procedure TimerProc();
var
  hCurWin : HWND;
  ptCursor : TPOINT;
  intPwLen : integer;
  strPassword : string;
begin
  // Retrieve the position of the mouse cursor
  GetCursorPos(ptCursor);
  // Get the handle of the window underneath the mouse cursor
  hCurWin := WindowFromPoint(ptCursor);
  // Exit if this is not a different window handle than
  // from the last time around
  if hCurWin = hLastWin then Exit;
  // Set the old handle equal to the current handle
  // (for the next time around)
  hLastWin := hCurWin;
  // Exit if no password character exists (i.e. not a password field)
  if SendMessage(hLastWin, EM_GETPASSWORDCHAR, 0, 0) = 0 then Exit;
  // Retrieve the length of the password
  intPwLen := SendMessage(hCurWin, WM_GETTEXTLENGTH, 0, 0) + 1;
  // Allocate enough space in the string buffer for the password
  SetLength(strPassword, intPwLen);
  // Copy the password into the string buffer
  SendMessage(hLastWin, WM_GETTEXT, intPwLen,
              LongInt(@strPassword[1]));
  // Set the text of our Edit control as the retrieved password
  SetWindowText(hEdit, PChar(strPassword));
end;


// WindowProc - processes all incoming window messages
function WindowProc(HWND, uMsg,	wParam,	lParam: integer): integer; stdcall;
begin

  // Pass the message to the default windows procedure
  Result := DefWindowProc(HWND, uMsg,	wParam,	lParam);

  // Here we will handle specific messages
  case uMsg of
    WM_COMMAND:
      // Process buton clicks
      begin
        // If the user clicked the About button...
        if lParam = hBtnAbt then ShowAboutProc();
        // If the user clicked the Help button...
        if lParam = hBtnHlp then ShowHelpProc();
      end;
    WM_TIMER:
      // Call the procedure to process the Timer message
      TimerProc();
    WM_DESTROY:
      // Call the procedure to handle the ending
      // of the application
      EndProgramProc();
  end;
end;



// The application itself
begin

  // Get the application instance
  hInst := GetModuleHandle(nil);
  
  // Define the attributes of our window class
  with wClass do
    begin
      style := CS_CLASSDC or CS_PARENTDC;   // class style
      lpfnWndProc := @WindowProc;           // pointer to our window procedure
      hInstance := hInst;                   // instance that our class is within
      hIcon := LoadIcon(hInst, 'MAINICON'); // the class icon
      hCursor := LoadCursor(0, IDC_ARROW);  // the class cursor
      hbrBackground := COLOR_BACKGROUND;    // class background color
      lpszClassName := 'NF_PW_REVEALER';    // the class name
    end;

  // Register our window class
  RegisterClass(wClass);

  // Retrieve the width and height of the screen area so
  // we can center our form on the screen
  intScrX := GetSystemMetrics(SM_CXSCREEN);
  intScrY := GetSystemMetrics(SM_CYSCREEN);

  // Create the main window
  hMainWin := CreateWindowEx(

      WS_EX_WINDOWEDGE or WS_EX_TOPMOST,  // extended window style
      'NF_PW_REVEALER',                   // registered class name
      'Password Revealer',                // window name
      WS_CAPTION or WS_SYSMENU or         // window style
        WS_MINIMIZEBOX or WS_VISIBLE,
      (intScrX - 266) div 2,              // window position (X, Y)
        (intScrY - 150) div 2,
      266, 150,                           // window width, height
      0,                                  // parent window handle
      0,                                  // menu handle
      hInst,                              // application instance handle
      nil                                 // pointer to window-creation data

      );


  // Create a font handle that we will use as the font
  // for our application's controls
  hFont := CreateFont(

      -12,                                // font height
      0,                                  // font width (0 will yield a 'closest match value')
      0,                                  // angle of escapement
      0,                                  // base-line angle
      FW_NORMAL,                          // font weight
      0,                                  // italic
      0,                                  // underlined
      0,                                  // strikethrough
      DEFAULT_CHARSET,                    // character set
      OUT_DEFAULT_PRECIS,                 // output precision
      CLIP_DEFAULT_PRECIS,                // clipping precision
      DEFAULT_QUALITY,                    // output quality
      DEFAULT_PITCH or FF_DONTCARE,       // font pitch
      'MS Sans Serif'                     // font typeface

      );


  // Create a raised bevel, just for cosmetic effects
  hBevel := CreateWindow(

      'Static',                           // class name
      '',                                 // window text/name
      WS_VISIBLE or WS_CHILD              // window style
        or WS_DLGFRAME,
      8, 8,                               // window position (X, Y)
      244, 80,                            // window width, height
      hMainWin,                           // parent window handle
      0,                                  // menu handle
      hInst,                              // application instance handle
      nil                                 // pointer to window-creation data

      );

  // Create a label
  hLabel := CreateWindow(

      'Static',
      'Revealed Password:',
      WS_VISIBLE or WS_CHILD or SS_LEFT,
      18, 20,
      220, 20,
      hMainWin,
      0,
      hInst,
      nil

      );

  // Create an Edit control
  hEdit:=CreateWindowEx(

      WS_EX_CLIENTEDGE,
      'Edit',
      '',
      WS_VISIBLE or WS_CHILD or ES_LEFT,
      18, 38,
      224, 20,
      hMainWin,
      0,
      hInst,
      nil

      );

  // Create a button - our About button
  hBtnAbt := CreateWindow(

      'Button',
      'About',
      WS_VISIBLE or WS_CHILD or
        BS_PUSHLIKE or BS_TEXT,
      177, 94,
      75, 25,
      hMainWin,
      0,
      hInst,
      nil

      );

  // Create another button - our Help button
  hBtnHlp := CreateWindow(

      'Button',
      'Help',
      WS_VISIBLE or WS_CHILD or
        BS_PUSHLIKE or BS_TEXT,
      96, 94,
      75, 25,
      hMainWin,
      0,
      hInst,
      nil

      );

  // Set the font for our controls
  SendMessage(hLabel, WM_SETFONT, hFont, 0);
  Sendmessage(hEdit, WM_SETFONT, hFont, 0);
  Sendmessage(hBtnHlp, WM_SETFONT, hFont, 0);
  Sendmessage(hBtnAbt, WM_SETFONT, hFont, 0);

  // Update our window's client area
  UpdateWindow(hMainWin);

  // Create a Timer and set its timeout value
  // (interval) to 25 milliseconds
  SetTimer(hMainWin, 0, 25, nil);

  // Below is the main message loop that our application will
  // use to retrieve messages from the incoming message queue
  while GetMessage(Msg, hMainWin, 0, 0) do
    begin
      // Convert any keyboard messages to WM_CHAR
      TranslateMessage(Msg);
      // Pass the message to our WindowProc function for processing
      DispatchMessage(Msg);
    end;
end.

